/*****************************************************************************
*  Copyright 2008 Broadcom Corporation.  All rights reserved.
*
*  Unless you and Broadcom execute a separate written software license
*  agreement governing use of this software, this software is licensed to you
*  under the terms of the GNU General Public License version 2, available at
*  http://www.gnu.org/licenses/old-license/gpl-2.0.html (the "GPL").
*
*  Notwithstanding the above, under no circumstances may you combine this
*  software in any way with any other Broadcom software provided under a
*  license other than the GPL, without Broadcom's express prior written
*  consent.
*
*****************************************************************************/

#include <common.h>
#include <command.h>

#if defined(CONFIG_CMD_BCM_DLMODE)

extern int do_reset (cmd_tbl_t *, int, int, char *[]);
extern int do_nand (cmd_tbl_t *, int, int, char *[]);
extern int do_go (cmd_tbl_t *, int, int, char *[]);
/*
 * Command dlmode: download mode.
 */

int cleanfs(cmd_tbl_t *cmdtp){
	char *local_args[4];
	local_args[0] = "nand";
	local_args[1] = "erase";
	local_args[2] = CFG_BCM_ROOTFS_ADDR;
	local_args[3] = CFG_BCM_ROOTFS_SIZE;
	return do_nand(cmdtp, 0, 4, local_args);
}

int do_dlmode(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	char *cmd;
	int ret;

	cmd = argv[1];	
	if (strcmp(cmd, "usb") == 0) {
		char *local_args[5];
		char *go_args[2];
		/* Load the image from nand to ram */
		local_args[0] = "nand";
		local_args[1] = "read";
		local_args[2] = CFG_BCM_BOOT2_LOADADDR;
		local_args[3] = CFG_BCM_BOOT2_OFFSET;
		local_args[4] = CFG_BCM_BOOT2_SIZE;
		ret = do_nand(cmdtp, 0, 5, local_args);

		if (ret) {
			printf("Error loading image.\n");
			return ret;
		}		
		
		if ( argc < 3 || strcmp(argv[2], "keepfs")) {
			ret = cleanfs(cmdtp);
			if(ret) {
				printf("Error erasing rootfs.\n");
				return ret;
			}
		}
		
		/* Turn on USB Mode */
		REG_PUMR = REG_PUMR_USB_DLMODE;
		/* Execute the program */

		go_args[0] = "go";
		go_args[1] = CFG_BCM_BOOT2_LOADADDR;

		printf("Entering dlmode, ready to get images...\n");
		ret = do_go(cmdtp, 0, 2, go_args);		
		
		/* Ends here, should not return! */
		printf("Error loading USB Download Driver\n");


	} else {

		REG_PUMR = REG_PUMR_UART_DLMODE;
		
		if ( argc < 2 || (argc == 2 && strcmp(cmd, "keepfs"))
			|| (argc == 3 && strcmp(argv[2], "keepfs"))) {
			ret = cleanfs(cmdtp);
			if(ret) {
				printf("Error erasing rootfs.\n");
				return ret;
			}
		}
		
		printf("Entering dlmode, ready to get images...\n");
		ret = do_reset(cmdtp, 0, 0, NULL); 
		
		/* Should not get here */
		printf("Error, could not reset board.\n");
	}
	
	return ret;
}

U_BOOT_CMD(
	dlmode,	3,	1,	do_dlmode,
	"dlmode	- Download Mode\n",
	"dlmode [mode] [keepfs]\n"
	"dlmode uart [keepfs]	- BROM UART Download Mode (Default)\n"
	"dlmode usb [keepfs]	- USB Download Mode\n"
	"	keepfs - don't delete fs partition\n"
	);

#endif
