/*****************************************************************************
*  Copyright 2004 - 2008 Broadcom Corporation.  All rights reserved.
*
*  Unless you and Broadcom execute a separate written software license
*  agreement governing use of this software, this software is licensed to you
*  under the terms of the GNU General Public License version 2, available at
*  http://www.gnu.org/copyleft/gpl.html (the "GPL").
*
*  Notwithstanding the above, under no circumstances may you combine this
*  software in any way with any other Broadcom software provided under a
*  license other than the GPL, without Broadcom's express prior written
*  consent.
*
*****************************************************************************/

/*                                                            
 * u-boot/board/Broadcom/BCM4760/sdhci.c
 * SD/SDHC/MMC storage device driver.
 */
#include <common.h>
#include <asm/arch/bcm4760.h>
#include <sdhci.h>
#include <asm/arch/sdhci.h>

//#define HCDEBUG

#ifdef CONFIG_CMD_SDHCI

DECLARE_GLOBAL_DATA_PTR;

/*
 * Transfer definitions
 */
#define DMA_MAX_BLOCKS		512
#define BLOCK_SIZE			512


/*
 * Local variables
 */
static int					sdhci_ready = 0;
static int					sdhci_hc_card = 0;

static ulong				sdhci_base;
static int					sdhci_rca;
static block_dev_desc_t*	sdhci_dev;
static block_dev_desc_t		sdhci_dev_desc[CFG_SDHCI_MAXDEVICE];
static sdhci_info_t sdhci_info = {0};

/*
 * SD/MMC register access
 */
#define sdhci_breg(offset)	((volatile uchar*)(sdhci_base + offset))
#define sdhci_wreg(offset)	((volatile ushort*)(sdhci_base + offset))
#define sdhci_reg(offset)	((volatile uint32_t*)(sdhci_base + offset))
#define sdhci_lreg(offset)	((volatile ulong*)(sdhci_base + offset))

/*
 * CSD/CID packed bit extraction
 */
#define UNSTUFF_BITS(resp,start,size)								\
		({															\
			const int __size = size;								\
			const u32 __mask = (__size < 32 ? 1 << __size : 0) - 1;	\
			const int __off = (start-8) >> 5;						\
			const int __shft = (start-8) & 31;						\
			u32 __res;												\
																	\
			__res = resp[__off] >> __shft;							\
			if (size + __shft > 32)									\
				__res |= resp[__off+1] << ((32 - __shft) % 32);		\
			__res & __mask;											\
		})


/*
 * Interrupt register access
 */
static inline uint32_t sdhci_get_int(void)
{
	return *sdhci_reg(SDHCI_INT_STS_LOFF);
}
static inline uint32_t sdhci_get_exp_int(ulong exp_int)
{
	return (sdhci_get_int() & exp_int);
}
static inline void sdhci_clr_int(uint32_t mask)
{
	*sdhci_reg(SDHCI_INT_STS_LOFF) = mask;
}
static inline void sdhci_clr_all_int(void)
{
	sdhci_clr_int(sdhci_get_int());
}

#ifdef HCDEBUG
static void dumpHCReg(void) 
{
	printf(": ============== REGISTER DUMP ==============\n");
	printf("Sys addr: 0x%08X | Version:  0x%08X\n",
		   (unsigned)*sdhci_reg(SDHCI_SDMA_SYS_ADDR_LOFF),
		   *sdhci_wreg(SDHCI_HOST_CTRLR_VERSION_WOFF));
	printf("Blk size: 0x%08X | Blk cnt:  0x%08X\n",
		   *sdhci_wreg(SDHCI_BLK_SIZE_WOFF),
		   *sdhci_wreg(SDHCI_BLK_CNT_WOFF));
	printf("Argument: 0x%08X | Trn mode: 0x%08X\n",
		   (unsigned)*sdhci_reg(SDHCI_ARGUMENT_LOFF),
		   *sdhci_wreg(SDHCI_XFER_MODE_WOFF));
	printf("Present:  0x%08X | Host ctl: 0x%08X\n",
		   (unsigned)*sdhci_reg(SDHCI_PRESENT_STATE_LOFF),
		   *sdhci_breg(SDHCI_HOST_CTRL_BOFF));
	printf("Power:    0x%08X | Blk gap:  0x%08X\n",
		   *sdhci_breg(SDHCI_POWER_CTRL_BOFF),
		   *sdhci_breg(SDHCI_BLOCK_GAP_CTRL_BOFF));
	printf("Wake-up:  0x%08X | Clock:    0x%08X\n",
		   *sdhci_breg(SDHCI_WAKEUP_CTRL_BOFF),
		   *sdhci_breg(SDHCI_CLOCK_CTRL_WOFF));
	printf("Timeout:  0x%08X | Int stat: 0x%08X\n",
		   *sdhci_breg(SDHCI_TIMEOUT_CTRL_BOFF),
		   (unsigned)*sdhci_reg(SDHCI_INT_STS_LOFF));
	printf("Int enab: 0x%08X | Sig enab: 0x%08X\n",
		   (unsigned)*sdhci_reg(SDHCI_INT_STS_EN_LOFF),
		   (unsigned)*sdhci_reg(SDHCI_INT_SIG_EN_LOFF));
	printf("AC12 err: 0x%08X | Slot int: 0x%08X\n",
		   *sdhci_wreg(SDHCI_ACMD12_ERR_STS_WOFF),
		   *sdhci_wreg(SDHCI_SLOT_INT_STS_WOFF));
	printf("Caps:     0x%08X | Max curr: 0x%08X\n",
		   (unsigned)*sdhci_reg(SDHCI_CAPABILITIES_LOFF),
		   (unsigned)*sdhci_reg(SDHCI_MAX_CURRENT_CAPABILITIES_LOFF));
	printf("sdhci_base %X\n", sdhci_base);
}
#endif

/*
 * Perform a reset
 */
static void sdhci_reset(u8 mask)
{
	volatile ulong tout;

	/*
	 * Start the requested reset
	 */
	*sdhci_breg(SDHCI_SW_RESET_BOFF) = mask;
	udelay(100);

	/*
	 * hw clears the bit when it's done, wait a max of 100ms
	 */
	for (tout = 100000;
		 *sdhci_breg(SDHCI_SW_RESET_BOFF) & mask;
		 tout--)
	{
		if (tout == 0)
		{
			puts("SDHCI: reset never completed.\n");
			break;
		}
		udelay(1);
	}
}

/*
 * Conditional reset on CMD and DATA. Use algorithm for
 * error recovery from SD Host Controller Standard
 * Specification Version 2.00.
 */
static void sdhci_cond_reset_cmd_data(uint32_t ints)
{
	if(ints & ((SDHCI_CMD_TIMEOUT_ERR |
				SDHCI_CMD_CRC_ERR |
				SDHCI_CMD_END_BIT_ERR |
				SDHCI_CMD_INDEX_ERR) << 16))
	{
		sdhci_reset(SDHCI_SW_RESET_CMDLINE);
	}

	if(ints & ((SDHCI_DATA_TIMEOUT_ERR |
				SDHCI_DATA_CRC_ERR |
				SDHCI_DATA_END_BIT_ERR) << 16))
	{
		sdhci_reset(SDHCI_SW_RESET_DATLINE);
	}

	if(!(ints & (SDHCI_DATA_TIMEOUT_ERR << 16)))
	{
		udelay(50);
	}
}

/*
 * Wait for a specified interrupt
 */
static int sdhci_pull_for_int(uint32_t exp_int)
{
	uint32_t ints;
	ulong tout = 0;

	while (sdhci_get_exp_int(exp_int) != exp_int)
	{
		if (tout >= 500)
		{
			return(1);
		}
		tout++;
		udelay(SDHCI_TIMEOUT);
	}
	sdhci_clr_int(exp_int);

	if ((ints = sdhci_get_exp_int(0xFFFF0000)) != 0)
	{
		printf("Error interrupt happened: 0x%x.\n", ints);
		sdhci_clr_int(ints);
		sdhci_cond_reset_cmd_data(ints);
	}
	return(0);
}

/*
 * Extract Command/Response flags based on spec
 */
static int sdhci_cmd_flags(int cmd_flags)
{
	int flags;

	if ((cmd_flags & MMC_RSP_136) && (cmd_flags & MMC_RSP_BUSY))
	{
		puts("Unsupported response type\n");
		return -1;
	}

	if (!(cmd_flags & MMC_RSP_PRESENT))
		flags = SDHCI_CMD_RESP_NONE;
	else if (cmd_flags & MMC_RSP_136)
		flags = SDHCI_CMD_RESP_LONG;
	else if (cmd_flags & MMC_RSP_BUSY)
		flags = SDHCI_CMD_RESP_SHORT_BUSY;
	else
		flags = SDHCI_CMD_RESP_SHORT;

	if (cmd_flags & MMC_RSP_CRC)
		flags |= SDHCI_CMD_CRC_CHK_ENABLE;
	if (cmd_flags & MMC_RSP_OPCODE)
		flags |= SDHCI_CMD_INDEX_CHK_ENABLE;

	return flags;
}


/*
 * Send a command and return immediately after sending the request
 */
static void sdhci_send_cmd(struct mmc_command* cmd)
{
	uint32_t cmd_value;
	int32_t flags = sdhci_cmd_flags(cmd->flags);
	if (flags == -1)
	{
		cmd->error = MMC_ERR_INVALID;
		return;
	}

	*sdhci_reg(SDHCI_ARGUMENT_LOFF)			= cmd->arg;

	cmd_value = ((cmd->opcode			<< 24) |
				 (cmd->dp				<< 21) |
				 (flags				<< 16) |
				 (cmd->dd				<< 4)  |
				 (cmd->ac12			<< 2)  |
				 (cmd->dma_ena));
	*sdhci_reg(SDHCI_CMD_XFER_MODE_LOFF)	= cmd_value;
}

/*
 * Send a command and return immediately after sending the request
 */
static int sdhci_send_cmd_and_retry(struct mmc_command* cmd, uint32_t retry_count, ulong wait_status)
{
	do
	{
		sdhci_send_cmd(cmd);
		if (!sdhci_pull_for_int(wait_status))
		{
			return 0;
		}

		sdhci_clr_all_int();
	} while (--retry_count != 0);

	return 1;
}

#if 1
static int32_t waitfor_data_transfer_done(void)
{
	uint32_t ints;
	uint32_t tout = 0;

	for (ints = sdhci_get_int();
		 !(ints & SDHCI_XFER_COMPLETE);
		 ints = sdhci_get_int())
	{
		if (ints & 0xFFFF0000)
		{
			sdhci_clr_int(ints);
			return (ints >> 16);
		}
		if (tout >= 100000)
		{
			return -1;
		}
		udelay(SDHCI_TIMEOUT);
		tout++;
	}
	sdhci_clr_int(ints);
	return 0;
}
#endif

/*
 * Check the response to see if it is what we expect
 */
static int sdhci_chk_rsp(ulong exp_resp)
{
	volatile ulong rsp;

	rsp = *sdhci_reg(SDHCI_RESPONSE10_LOFF);
	return((rsp != exp_resp) ? 0 : 1);
}


/*
 * Check the capabilities for required functionality
 */
static int sdhci_chk_capabilities(void)
{
	ulong capable;

	/*
	 * Capabilities
	 * 05:00	Timeout Frequency
	 * 07		Timeout Unit (1=MHZ)
	 * 13:08	Base Clock Frequency
	 * 17:16	Max Block (0=512, 1=1024, 2=2048, 3=4096)
	 * 18		Extended Media Bus Support
	 * 19		ADMA2 Support
	 * 21		High Speed Support
	 * 22		SDMA Support
	 * 23		Suspend/Resume
	 * 24		3.3 Volt
	 * 25		3.0 Volt
	 * 26		1.8 Volt
	 * 27		Interrupt Mode
	 * 28		64 Bit Mode
	 * 29		SPI Mode
	 * 30		SPI Block Mode
	 */
	capable = *sdhci_reg(SDHCI_CAPABILITIES_LOFF);
	{
		ulong sdma;
		sdma = (capable >> 22) & 1;

		if (sdma == 0)
		{
			return(0);
		}
	}
	return(1);
}

static int send_abort_cmd(void)
{
	struct mmc_command mcmd;

	sdhci_reset(SDHCI_SW_RESET_CMDLINE);
	sdhci_reset(SDHCI_SW_RESET_DATLINE);

	memset((uint8_t*)&mcmd, 0, sizeof(mcmd));
	mcmd.opcode = MMC_STOP_TRANSMISSION;
	mcmd.arg = 0;
	mcmd.flags = MMC_RSP_R1;
	sdhci_send_cmd(&mcmd);
	if (sdhci_pull_for_int(SDHCI_CMD_COMPLETE))
	{
		return 1;
	}
	return 0;
}

static int send_abort_cmd_noresets(void)
{
	struct mmc_command mcmd;

	memset((uint8_t*)&mcmd, 0, sizeof(mcmd));
	mcmd.opcode = MMC_STOP_TRANSMISSION;
	mcmd.arg = 0;
	mcmd.flags = MMC_RSP_R1;
	sdhci_send_cmd(&mcmd);
	if (sdhci_pull_for_int(SDHCI_CMD_COMPLETE))
	{
		return 1;
	}
	return 0;
}

static int
/****************************************************/
sdhci_mblock_read(int blkcnt, uchar *dst, ulong src)
/****************************************************/
{
	int flags;
	int pass_blkcnt;
	ulong dst_l = (ulong)dst;
	uint32_t cmd_value;

#ifdef HCDEBUG
	debug("sdhci_mblock_read dst %lx src %lx \n", dst_l, src);
#endif

#if CONFIG_ENABLE_DCACHE
	{
		void arm1136_dcache_inv(ulong start, ulong end);
		arm1136_dcache_inv(dst_l, dst_l + (blkcnt*BLOCK_SIZE));
	}
#endif

	/*
	 * Start with clear interrupts
	 */
	sdhci_clr_all_int();
	
/*
	 * Initial maximum transfer size needs to be either the set maximum or
	 * the amount of blocks to get to the next DMA boundary
	 */
	if (dst_l & ((DMA_MAX_BLOCKS * BLOCK_SIZE) - 1))
	{
		pass_blkcnt = (dst_l & ((DMA_MAX_BLOCKS * BLOCK_SIZE) - 1)) >> 9;
	}
	else
	{
		pass_blkcnt = DMA_MAX_BLOCKS;
	}

	/*
	 * Continue as long as there is more to transfer... also make sure to
	 * attempt the maximum transfer for each pass after the first one that
	 * gets us on the DMA boundary
	 */
	for (; blkcnt; pass_blkcnt=DMA_MAX_BLOCKS)
	{
		/*
		 * Determine the maximum size of this transfer.
		 */
		if (blkcnt < pass_blkcnt)
		{
			pass_blkcnt = blkcnt;
		}

		/*
		 * Reduce this passes block count from the total
		 */
		blkcnt -= pass_blkcnt;

		/*
		 * Send Command (READ_MULTIPLE-18)
		 */
		flags = sdhci_cmd_flags(MMC_CMD_AC | MMC_RSP_R1);

		*sdhci_reg(SDHCI_SDMA_SYS_ADDR_LOFF) = dst_l;
		*sdhci_wreg(SDHCI_BLK_SIZE_WOFF) = (SDHCI_HOST_SDMA_BUFFER_SIZE_512K |
											sdhci_dev->blksz);
		*sdhci_wreg(SDHCI_BLK_CNT_WOFF)		= pass_blkcnt;
		*sdhci_reg(SDHCI_ARGUMENT_LOFF)		= ((sdhci_hc_card)
											   ? (src >> 9) : (src));
		cmd_value = ((MMC_READ_MULTIPLE_BLOCK	<< 24)	|
					 (SDHCI_DATA_PRESENT		<< 16)	|
					 (flags						<< 16)	|
					 (SDHCI_MULTI_BLK)					|
					 (SDHCI_DATA_XFER_READ)				|
					 (SDHCI_ACMD12_ENABLE)				|
					 (SDHCI_BLK_CNT_ENABLE)				|
					 (SDHCI_DMA_ENABLE));
		*sdhci_reg(SDHCI_CMD_XFER_MODE_LOFF)	= cmd_value;

		/*
		 * Move the pointers to the next possible block
		 */
		dst_l += (pass_blkcnt * BLOCK_SIZE);
		src  += (pass_blkcnt * BLOCK_SIZE);

		/*
		 * Wait for cmd complete interrupt
		 */
		if (sdhci_pull_for_int(SDHCI_CMD_COMPLETE))
		{
			printf ("mblock_read: Interrupt timeout MMC_READ_MULTIPLE_BLOCK - CC [command=0x%x]\n", cmd_value);
			return -1;
		}
		sdhci_clr_int(SDHCI_CMD_COMPLETE);

		/*
		 * Wait for transfer complete interrupt
		 */
		for (;;)
		{
			/*
			 * Get the interrupt status
			 */
			uint32_t ints = sdhci_get_int();

			/*
			 * Clear the interrupt status
			 */
			sdhci_clr_int(ints);

			/*
			 * Report any errors
			 */
			if (ints & ((SDHCI_CMD_TIMEOUT_ERR |
						SDHCI_CMD_CRC_ERR |
						SDHCI_CMD_END_BIT_ERR |
						SDHCI_CMD_INDEX_ERR |
						SDHCI_DATA_CRC_ERR |
						SDHCI_DATA_END_BIT_ERR |
						SDHCI_CURRENT_LIMIT_ERR |
						SDHCI_ACMD12_ERR |
						SDHCI_ADMA_ERR |
						SDHCI_TARGET_RESP_ERR) << 16)) 
			{
				printf("mblock_read: Error interrupt happened: 0x%x [command=0x%x].\n", ints, cmd_value);
				sdhci_cond_reset_cmd_data(ints);
				(void)send_abort_cmd_noresets();
				return -1;
			}

			/*
			 * Done with the transfer
			 */
			if (ints & SDHCI_XFER_COMPLETE)
			{
				break;
			}

			if (ints & (SDHCI_DATA_TIMEOUT_ERR << 16))
			{
				printf("Error interrupt happened: 0x%x.\n", ints);
				sdhci_cond_reset_cmd_data(ints);
				(void)send_abort_cmd_noresets();
				return -1;
			}
			
			/*
			 * Restart the DMA operation on DMA boundaries
			 */
			else if (ints & SDHCI_DMA_INT)
			{
				*sdhci_reg(SDHCI_SDMA_SYS_ADDR_LOFF) = 
				  *sdhci_reg(SDHCI_SDMA_SYS_ADDR_LOFF);
			}
			
			/*
			 * Still waiting for the expected interrupt
			 */
			else
			{
				udelay(50);
			}
		}
	}
	return 0;
}

static int
/****************************************************/
sdhci_mblock_write(int blkcnt, ulong dst, uchar *src)
/****************************************************/
{
	int flags;
	int pass_blkcnt;
	ulong src_l = (ulong)src;
	uint32_t cmd_value;

#ifdef HCDEBUG
	debug("sdhci_mblock_write dst %lx src %lx \n", dst, src_l);
#endif

	/*
	 * Start with clear interrupts
	 */
	sdhci_clr_all_int();

	/*
	 * Initial maximum transfer size needs to be either the set maximum or
	 * the amount of blocks to get to the next DMA boundary
	 */
	if (src_l & ((DMA_MAX_BLOCKS * BLOCK_SIZE) - 1))
	{
		pass_blkcnt = (src_l & ((DMA_MAX_BLOCKS * BLOCK_SIZE) - 1)) >> 9;
	}
	else
	{
		pass_blkcnt = DMA_MAX_BLOCKS;
	}

	/*
	 * Continue as long as there is more to transfer... also make sure to
	 * attempt the maximum transfer for each pass after the first one that
	 * gets us on the DMA boundary
	 */
	for (; blkcnt; pass_blkcnt=DMA_MAX_BLOCKS)
	{
		/*
		 * Determine the maximum size of this transfer.
		 */
		if (blkcnt < pass_blkcnt)
		{
			pass_blkcnt = blkcnt;
		}

		/*
		 * Reduce this passes block count from the total
		 */
		blkcnt -= pass_blkcnt;

		/*
		 * Send Command (WRITE_MULTIPLE-25)
		 */
		flags = sdhci_cmd_flags(MMC_CMD_AC | MMC_RSP_R1);

		*sdhci_reg(SDHCI_SDMA_SYS_ADDR_LOFF) = src_l;
		*sdhci_wreg(SDHCI_BLK_SIZE_WOFF) = (SDHCI_HOST_SDMA_BUFFER_SIZE_512K |
											sdhci_dev->blksz);
		*sdhci_wreg(SDHCI_BLK_CNT_WOFF)		= pass_blkcnt;
		*sdhci_reg(SDHCI_ARGUMENT_LOFF)		= ((sdhci_hc_card)
											   ? (dst >> 9) : (dst));
		cmd_value = ((MMC_WRITE_MULTIPLE_BLOCK	<< 24)	|
					 (SDHCI_DATA_PRESENT		<< 16)	|
					 (flags						<< 16)	|
					 (SDHCI_MULTI_BLK)					|
					 (SDHCI_DATA_XFER_WRITE)			|
					 (SDHCI_ACMD12_ENABLE)				|
					 (SDHCI_BLK_CNT_ENABLE)				|
					 (SDHCI_DMA_ENABLE));
		*sdhci_reg(SDHCI_CMD_XFER_MODE_LOFF)	= cmd_value;

		/*
		 * Move the pointers to the next possible block
		 */
		dst += (DMA_MAX_BLOCKS * BLOCK_SIZE);
		src_l += (DMA_MAX_BLOCKS * BLOCK_SIZE);

		/*
		 * Wait for cmd complete interrupt
		 */
		if (sdhci_pull_for_int(SDHCI_CMD_COMPLETE))
		{
			puts("SDHCI: Interrupt timeout MMC_WRITE_MULTIPLE_BLOCK - CC.\n");
			return -1;
		}
		sdhci_clr_int(SDHCI_CMD_COMPLETE);

		/*
		 * Wait for transfer complete interrupt
		 */
		for (;;)
		{
			/*
			 * Get the interrupt status
			 */
			uint32_t ints = sdhci_get_int();

			/*
			 * Clear the interrupt status
			 */
			sdhci_clr_int(ints);

			/*
			 * Report any errors
			 */
			if (ints & ((SDHCI_CMD_TIMEOUT_ERR |
						SDHCI_CMD_CRC_ERR |
						SDHCI_CMD_END_BIT_ERR |
						SDHCI_CMD_INDEX_ERR |
						SDHCI_DATA_CRC_ERR |
						SDHCI_DATA_END_BIT_ERR |
						SDHCI_CURRENT_LIMIT_ERR |
						SDHCI_ACMD12_ERR |
						SDHCI_ADMA_ERR |
						SDHCI_TARGET_RESP_ERR) << 16))
			{
				printf("Error interrupt happened: 0x%x.\n", ints);
				sdhci_cond_reset_cmd_data(ints);
				(void)send_abort_cmd_noresets();
				return -1;
			}

			/*
			 * Done with the transfer
			 */
			if (ints & SDHCI_XFER_COMPLETE)
			{
				break;
			}

			if (ints & (SDHCI_DATA_TIMEOUT_ERR << 16))
			{
				printf("Error interrupt happened: 0x%x.\n", ints);
				sdhci_cond_reset_cmd_data(ints);
				(void)send_abort_cmd_noresets();
				return -1;
			}
#if 0
			/*
			 * Restart the DMA operation on DMA boundaries
			 */
			else if (ints & SDHCI_DMA_INT)
			{
				*sdhci_reg(SDHCI_SDMA_SYS_ADDR_LOFF) = 
				  *sdhci_reg(SDHCI_SDMA_SYS_ADDR_LOFF);
			}
#endif
			/*
			 * Still waiting for the expected interrupt
			 */
			else
			{
				udelay(50);
			}
		}
	}
	return 0;
}

static uint32_t get_size_value(char* value_ptr)
{
	uint32_t value;
	char* end_ptr;

	if (value_ptr == NULL)
	{
		value = 0;
	}
	else
	{
		value = simple_strtoul(value_ptr, &end_ptr, 10);
		if (*end_ptr)
		{
			if (*end_ptr == 'K' || *end_ptr == 'k')
			{
				value *= 1024;
			}
			else if (*end_ptr == 'M' || *end_ptr == 'm')
			{
				value *= (1024 * 1024);
			}
		}
	}
	return value;
}

uint32_t round_to_nearest_mb(uint32_t value)
{
	value += (1024 * 1024) - 1;
	value &= ~((1024 * 1024) - 1);
	return value;
}


/*****************************************************************************
 ** PUBLIC INTERFACE
 *****************************************************************************/
block_dev_desc_t* sdhci_get_dev(int dev_num)
{
	return((dev_num < 0 || dev_num >= CFG_SDHCI_MAXDEVICE)
		   ? NULL : &sdhci_dev_desc[dev_num]);
}

void sdhci_init(int dev_num)
{
	volatile uchar done;
	struct mmc_command mcmd;
	int tout = 0;
	int flag = 0;
	uint32_t sd_hcs;
	int if_type = IF_TYPE_SD;
	int sdhc_version2_struct;
	int sdhc_mmc_spec_vers;
	uint32_t retrycnt;
	uint16_t tmp16 = 0;
	ulong ints;
	uint8_t tran_speed=0;

	/*
	 * Setup global access for macros for this device
	 */
	{
		sdhci_dev = sdhci_get_dev(dev_num);
		if (dev_num == 0)
		{
			sdhci_base = HW_SDM0_BASE;
		}
		else if (dev_num == 1)
		{
			sdhci_base = HW_SDM1_BASE;
		}
		else if (dev_num == 2)
		{
			sdhci_base = HW_SDM2_BASE;
		}
		else
		{
			/*
			 * Can't handle this dev_num
			 */
			printf("SDHCI: Invalid device number (%d)\n", dev_num);
			return;
		}
	}

	sdhci_info.sys_buf = (uint8_t*)(((uint32_t)sdhci_info.sys_buffer + 15) &
			0xfffffff0);

	/*
	 * Fill in the device description for this device
	 */
	{
		memset(sdhci_dev, 0, sizeof(block_dev_desc_t));
		sdhci_dev->if_type			= IF_TYPE_UNKNOWN;
		sdhci_dev->dev				= dev_num;
		sdhci_dev->part_type		= PART_TYPE_UNKNOWN;
		//sdhci_dev->target			= 0;
		//sdhci_dev->lun			= 0;
		sdhci_dev->type				= DEV_TYPE_UNKNOWN;
		//sdhci_dev->removable		= 0;
		//sdhci_dev->lba			= 0;
		sdhci_dev->blksz			= BLOCK_SIZE;
		sdhci_dev->block_read		= sdhci_read;
		sdhci_dev->block_write		= sdhci_write;
		sdhci_dev->priv				= (void*)sdhci_base;
	}

	/*
	 * Reset the device
	 */
	sdhci_reset(SDHCI_SW_RESET_ALL);
	sdhci_clr_all_int();
	*sdhci_breg(SDHCI_HOST_CTRL_BOFF) &= ~ (SDHCI_HIGH_SPEED_ENABLE |
											SDHCI_SD_8BIT_MODE |
											SDHCI_DATA_TRANSFER_WIDTH_4BIT);

	/*
	 * Give the card a moment to think about things
	 */
	udelay(50);

	/*
	 * Turn on Power
	 */
	*sdhci_breg(SDHCI_POWER_CTRL_BOFF) = (SDHCI_SDBUS_VOLTAGE_3_3V);
	udelay(1000);
	*sdhci_breg(SDHCI_POWER_CTRL_BOFF) = (SDHCI_SDBUS_POWER		|
										  SDHCI_SDBUS_VOLTAGE_3_3V);

	/*
	 * See if a card is present
	 */
	{
		ulong present_state = *sdhci_reg(SDHCI_PRESENT_STATE_LOFF);

		if (!(present_state & SDHCI_CARD_INSERTED))
		{
			puts("SDHCI: SD/MMC device not inserted.\n");
			return;
		}
	}

	/*
	 * Check Capabilities Register
	 */
	if (!sdhci_chk_capabilities())
	{
		puts("SDHCI: Incapable SD/MMC device.\n");
		return;
	}

	/*
	 * Set up clocks
	 * the F_od clock needs to be between 0 and 400khz.
	 * the F_pp clock needs to be between 26mhz and 52mhz.
	 */
	*sdhci_wreg(SDHCI_CLOCK_CTRL_WOFF) = (SDHCI_SD_CLOCK_DIV_256 |
										  SDHCI_INTERNAL_CLOCK_ENABLE);
	while (*sdhci_wreg(SDHCI_CLOCK_CTRL_WOFF) !=
		   (SDHCI_SD_CLOCK_DIV_256 |
			SDHCI_INTERNAL_CLOCK_ENABLE |
			SDHCI_INTERNAL_CLOCK_STABLE))
	{
		udelay(50);
	}

	*sdhci_wreg(SDHCI_CLOCK_CTRL_WOFF) = (SDHCI_SD_CLOCK_DIV_256 |
										  SDHCI_INTERNAL_CLOCK_ENABLE |
										  SDHCI_SD_CLOCK_ENABLE);
	while (*sdhci_wreg(SDHCI_CLOCK_CTRL_WOFF) !=
		   (SDHCI_SD_CLOCK_DIV_256 |
			SDHCI_INTERNAL_CLOCK_ENABLE |
			SDHCI_SD_CLOCK_ENABLE |
			SDHCI_INTERNAL_CLOCK_STABLE))
	{
		udelay(50);
	}

	// NOTE: there seems to be a hw bug that cause about a byte worth of
	// junk data to be sent out as soon as SD clock is enable in the above
	// step; this junk data is interpret as the Arasan controller as
	// issued cmd so inhibit cmd bit is set in the current state status;
	// since the cmd is incomplete, the device will not respond; the
	// controller should timed-out but it's not;
	// To fix this problem, we will reset both data and cmd lines which
	// will clear out cmd inhibit and interrupt register
	sdhci_reset(SDHCI_SW_RESET_CMDLINE);
	sdhci_reset(SDHCI_SW_RESET_DATLINE);

	// Set the timeout control register
	*sdhci_breg(SDHCI_TIMEOUT_CTRL_BOFF) = 0x0D;

	/*
	 * Enable interrupts, include all error enables even if they
	 * are currently reserved
	 */
	*sdhci_reg(SDHCI_INT_STS_EN_LOFF) = (SDHCI_CMD_COMPLETE			|
										 SDHCI_XFER_COMPLETE		|
										 SDHCI_DMA_INT			|
										 //SDHCI_BUFFER_WRITE_READY	|
										 //SDHCI_BUFFER_READ_READY	|
										 0x00ff0000);

#ifdef HCDEBUG
	dumpHCReg();
#endif

	/*
	 * Send Command (GO_IDLE-0) and wait for response
	 * Any State => IDLE
	 */
	memset(&mcmd, 0, sizeof(mcmd));
	mcmd.opcode = MMC_GO_IDLE_STATE;
	mcmd.flags = MMC_RSP_NONE | MMC_CMD_BC;
	sdhci_send_cmd(&mcmd);
#if 1
	for( tout=0 ; tout <= 100000 ; tout++ )
	{
		ints = sdhci_get_int();
		if(!(ints & SDHCI_CMD_COMPLETE))
		{
			continue;
		}
	}
	if(!(ints & SDHCI_CMD_COMPLETE))
	{
		puts("SDHCI: Interrupt timeout MMC_GO_IDLE_STATE.\n");
		return;
	}

#else
	if (sdhci_pull_for_int(SDHCI_CMD_COMPLETE))
	{
		puts("SDHCI: Interrupt timeout MMC_GO_IDLE_STATE.\n");
		return;
	}
#endif
	sdhci_clr_all_int();

	/*
	 * Check Capabilities Register Again
	 */
	if (!sdhci_chk_capabilities())
	{
		puts ("Incapable SD device after 2nd check.\n");
		return;
	}

	/* 
	 * Adding some delay here to make sure the flash card is powered up
	 * before sending CMD8
	 */	
//	udelay(100);

	retrycnt = 3;
	do
	{
		/*
		 * Send Command (SEND_IF_COND-8) and wait for response
		 * IDLE => IDLE (SD Version 2 cards only)
		 */
		memset(&mcmd, 0, sizeof(mcmd));
		mcmd.opcode = SD_SEND_IF_COND;
		mcmd.arg = 0x000001AA;
		mcmd.flags = MMC_RSP_R7 | MMC_CMD_BCR;
		mcmd.resp[0] = 0x000001AA;
		sdhci_send_cmd(&mcmd);
#if 1
		for( tout=0 ; tout <= 50000 ; tout++ )
		{
			ints = sdhci_get_int();
			if(ints & SDHCI_CMD_COMPLETE)
			{
				if(!sdhci_chk_rsp(mcmd.resp[0]))
				{
					printf("SDHCI: test pattern failed to compare for CMD8. resp => %08X\n", mcmd.resp[0]);
					return;
				}
				sd_hcs = 1;
				retrycnt=0;		/* force exit from outer loop also */
				break;
			}
			else if ((ints & (SDHCI_ERR_INT | (SDHCI_CMD_TIMEOUT_ERR << 16))) ==
					 (SDHCI_ERR_INT | (SDHCI_CMD_TIMEOUT_ERR << 16)))
			{
				sdhci_clr_int(ints);
				sdhci_reset(SDHCI_SW_RESET_CMDLINE);
				sdhci_reset(SDHCI_SW_RESET_DATLINE);
				retrycnt=0;		/* force exit from outer loop also */
				break;
			}

			udelay(SDHCI_TIMEOUT);
		}
#else /* This code should become obsolete and be deleted. */
		if (!sdhci_pull_for_int(SDHCI_CMD_COMPLETE))
		{
			if(!sdhci_chk_rsp(mcmd.resp[0]))
			{
				printf("SDHCI: test pattern failed to compare for CMD8. resp => %08\n", mcmd.resp[0]);
				return;
			}
			sd_hcs = 1;
			retrycnt=0;		/* force exit from outer loop also */
			break;
		}
		else
		{
			sdhci_clr_int(sdhci_get_exp_int((SDHCI_CMD_COMPLETE)        |
											(SDHCI_CMD_TIMEOUT_ERR << 16)));
			sdhci_reset(SDHCI_SW_RESET_CMDLINE);
			sdhci_reset(SDHCI_SW_RESET_DATLINE);
		}
#endif
	} while (retrycnt-- != 0);

	/*
	 * TODO: May want to check vltg range and set the range.
	 */

	
	sd_hcs = 1;
	tout = 0;
	flag = 0;
	while (tout <= (1000000/500))
	{
		/*
		* Send Command (APP_CMD-55) and wait for response
		* IDLE => IDLE (MMC not valid from IDLE)
		*/
		memset(&mcmd, 0, sizeof(mcmd));
		mcmd.opcode = MMC_APP_CMD;
		mcmd.arg = 0;
		mcmd.flags = MMC_RSP_R1 | MMC_CMD_AC;
		sdhci_send_cmd(&mcmd);
		if (sdhci_pull_for_int(SDHCI_CMD_COMPLETE))
		{

			/*
			* This is quite possibly here because we are talking to an MMC
			* card. Make sure we are in a clean state
			*/
			sdhci_clr_all_int();
			sdhci_reset(SDHCI_SW_RESET_CMDLINE);
			sdhci_reset(SDHCI_SW_RESET_DATLINE);
			udelay(100);

			/*
			* Send Command (SEND_OP_COND-1) and wait for response
			* IDLE => READY
			*/
			memset(&mcmd, 0, sizeof(mcmd));
			mcmd.opcode = MMC_SEND_OP_COND; // MMC_SEND_OP_COND;
			mcmd.arg = 0x00FF8000 | (2 << 29);
			mcmd.flags = MMC_RSP_R3 | MMC_CMD_BCR;
			mcmd.resp[0] = 0x00FF8000 | (2 << 29);
			sdhci_send_cmd(&mcmd);
		}
		else
		{
			sdhci_clr_all_int();

			/*
			* Send Command (APP_OP_COND-41) and wait for response
			* IDLE => READY (MMC not valid)
			*/
			memset(&mcmd, 0, sizeof(mcmd));
			mcmd.opcode = SD_APP_OP_COND;
			mcmd.arg = 0x00380000 | (sd_hcs << 30);
			mcmd.flags = MMC_RSP_R3 | MMC_CMD_BCR;
			mcmd.resp[0] = 0x00380000 | (sd_hcs << 30);
			sdhci_send_cmd(&mcmd);
		}

		if (sdhci_pull_for_int(SDHCI_CMD_COMPLETE))
		{
			if (tout <= (1000000/500))
			{
				sdhci_clr_int(ints);
				sdhci_reset(SDHCI_SW_RESET_CMDLINE);
				sdhci_reset(SDHCI_SW_RESET_DATLINE);
//				udelay(SDHCI_TIMEOUT);
				tout++;
				continue;
			}
			printf("SDHCI: Interrupt timeout MMC_SEND_OP_COND or SD_APP_OP_COND-41.\n");
			return;
		}

		if(mcmd.opcode == MMC_SEND_OP_COND)
			if_type = IF_TYPE_MMC;
		else if(mcmd.opcode == SD_APP_OP_COND)
			if_type = IF_TYPE_SD;

		/*
		* Determine if the card thinks we are done or if we need to wait
		*/
		done = ((*sdhci_reg(SDHCI_RESPONSE10_LOFF) & 0x80000000) ? 1 : 0);
		if (!done)
		{
			if (tout <= (1000000/500))
			{
				if (!flag)
				{
					printf("SDHCI: Waiting for device ready.");
					flag = 1;
				}
				else
				{
					printf(".");
				}
				udelay(SDHCI_TIMEOUT);
				tout++;

				continue;
			}
			else
			{
				sdhci_clr_int(ints);
				sdhci_reset(SDHCI_SW_RESET_CMDLINE);
				sdhci_reset(SDHCI_SW_RESET_DATLINE);
				if(flag)
					printf("busy for >1 second.\n");
				return;
			}
		}
		else
		{
			uint32_t voltages;

			if(flag)
				printf(" ready.\n");

			//if CCS bit is set then its HC card
			sdhci_hc_card = ((*sdhci_reg(SDHCI_RESPONSE10_LOFF) & 0x40000000)
							? 1 : 0);
	
			//make sure voltage modes are supported
			voltages = *sdhci_reg(SDHCI_RESPONSE10_LOFF) & 0x00ff8000;
			if (!(voltages & (0x00FF8000 & mcmd.resp[0])))
			{
				printf("SDHCI: supported voltage mismatch.\n");
				return;
			}
			break;
		}

		sdhci_clr_int(ints);
		sdhci_reset(SDHCI_SW_RESET_CMDLINE);
		sdhci_reset(SDHCI_SW_RESET_DATLINE);
	}
	sdhci_clr_all_int();

	/*
	 * Send Command (ALL_SEND_CID-2) and wait for response
	 * READY => IDENT
	 */
	memset(&mcmd, 0, sizeof(mcmd));
	mcmd.opcode = MMC_ALL_SEND_CID;
	mcmd.flags = MMC_RSP_R2 | MMC_CMD_BCR;
	sdhci_send_cmd(&mcmd);
	if (sdhci_pull_for_int(SDHCI_CMD_COMPLETE))
	{
		printf("SDHCI: Interrupt timeout MMC_ALL_SEND_CID.\n");
		return;
	}
	{
		struct mmc_cid cid;
		volatile ulong *resp;

		resp = sdhci_lreg(SDHCI_RESPONSE10_LOFF);

		/*
		 * SD doesn't currently have a version field so we will
		 * have to assume we can parse this.
		 */
		memset(&cid, 0, sizeof(cid));
		switch (if_type)
		{
			case IF_TYPE_MMC:
			{
				cid.manfid			= UNSTUFF_BITS(resp, 120, 8);
				cid.oemid			= UNSTUFF_BITS(resp, 104, 16);
				cid.prod_name[0]	= UNSTUFF_BITS(resp,  96, 8);
				cid.prod_name[1]	= UNSTUFF_BITS(resp,  88, 8);
				cid.prod_name[2]	= UNSTUFF_BITS(resp,  80, 8);
				cid.prod_name[3]	= UNSTUFF_BITS(resp,  72, 8);
				cid.prod_name[4]	= UNSTUFF_BITS(resp,  64, 8);
				cid.prod_name[5]	= UNSTUFF_BITS(resp,  56, 8);
				cid.hwrev			= UNSTUFF_BITS(resp,  52, 4);
				cid.fwrev			= UNSTUFF_BITS(resp,  48, 4);
				cid.serial			= UNSTUFF_BITS(resp,  16, 32);
				cid.month			= UNSTUFF_BITS(resp,  12, 4);
				cid.year			= UNSTUFF_BITS(resp,   8, 4) + 1997;
			}
			break;
			case IF_TYPE_SD:
			//case IF_TYPE_SDHC:
			{
				cid.manfid			= UNSTUFF_BITS(resp, 120, 8);
				cid.oemid			= UNSTUFF_BITS(resp, 104,16);
				cid.prod_name[0]	= UNSTUFF_BITS(resp,  96, 8);
				cid.prod_name[1]	= UNSTUFF_BITS(resp,  88, 8);
				cid.prod_name[2]	= UNSTUFF_BITS(resp,  80, 8);
				cid.prod_name[3]	= UNSTUFF_BITS(resp,  72, 8);
				cid.prod_name[4]	= UNSTUFF_BITS(resp,  64, 8);
				cid.hwrev			= UNSTUFF_BITS(resp,  60, 4);
				cid.fwrev			= UNSTUFF_BITS(resp,  56, 4);
				cid.serial			= UNSTUFF_BITS(resp,  24, 32);
				cid.year			= UNSTUFF_BITS(resp,  12, 8) + 2000;
				cid.month			= UNSTUFF_BITS(resp,   8, 4);
			}
			break;
		}
		sprintf(sdhci_dev->vendor,"Man %x Snr %x", cid.manfid, cid.serial);
		sprintf(sdhci_dev->product,"%s", cid.prod_name);
		sprintf(sdhci_dev->revision,"%d.%d", cid.hwrev, cid.fwrev);

#if 0
		printf("SD/MMC found. Card desciption is:\n");
		printf("Manufacturer ID = %lx\n", cid.manfid);
		printf("HW/FW Revision = %x %x\n",cid.hwrev, cid.fwrev);
		printf("Product Name = %s\n",cid.prod_name);
		printf("Serial Number = %lx\n", cid.serial);
		printf("Month = %d\n",cid.month);
		printf("Year = %d\n",cid.year);
#endif
	}
	sdhci_clr_all_int();

	/*
	 * Fill in the block size/count
	 * NOTE: I saw a note in another driver for this engine that said that we do
	 * not handle buffer size breaks...
	 */
	*sdhci_wreg(SDHCI_BLK_CNT_WOFF) = 1;
	*sdhci_wreg(SDHCI_BLK_SIZE_WOFF) = (SDHCI_HOST_SDMA_BUFFER_SIZE_512K |
										sdhci_dev->blksz);

	/*
	 * On MMC, this is a SET_RELATIVE_ADDR and we have to send a non-zero
	 * value in the RCA register... we will pick 1.
	 * On SD, this is a SEND_RELATIVE_ADDR and we have to send a zero in
	 * hopes of retrieving the actual value back from the card.
	 */
	sdhci_info.card_rca = sdhci_rca = (if_type == IF_TYPE_MMC) ? 1 : 0;

	/*
	 * Send Command (SET_RELATIVE_ADDR-3) and wait for response
	 * IDENT => STANDBY
	 */
	memset(&mcmd, 0, sizeof(mcmd));
	mcmd.opcode = MMC_SET_RELATIVE_ADDR;
	mcmd.arg = sdhci_rca << 16;
	if (if_type != IF_TYPE_MMC)
	{
		mcmd.flags = MMC_RSP_R6 | MMC_CMD_BCR;
	}
	else
	{
		mcmd.flags = MMC_RSP_R1 | MMC_CMD_BCR;
	}
	sdhci_send_cmd(&mcmd);
	if (sdhci_pull_for_int(SDHCI_CMD_COMPLETE))
	{
		printf("SDHCI: Interrupt timeout MMC_SET_RELATIVE_ADDR.\n");
		return;
	}
	else if (if_type != IF_TYPE_MMC)
	{
		volatile ulong resp;

		resp = (ulong)*sdhci_reg(SDHCI_RESPONSE10_LOFF);
		sdhci_info.card_rca = sdhci_rca = (uint16_t)(resp >> 16);
	}
	sdhci_clr_all_int();

	/*
	 * Send Command (SEND_CSD-9) and wait for response
	 * STANDBY => STANDBY
	 */
	memset(&mcmd, 0, sizeof(mcmd));
	mcmd.opcode = MMC_SEND_CSD;
	mcmd.arg = (sdhci_rca << 16);
	mcmd.flags = MMC_RSP_R2 | MMC_CMD_AC;
	sdhci_send_cmd(&mcmd);
	if (sdhci_pull_for_int(SDHCI_CMD_COMPLETE))
	{
		printf("SDHCI: Interrupt timeout MMC_SEND_CSD.\n");
		return;
	}
	{
		volatile ulong *resp;

		resp = sdhci_lreg(SDHCI_RESPONSE10_LOFF);

		if (if_type == IF_TYPE_MMC)
		{
			sdhc_version2_struct = 0;
			sdhc_mmc_spec_vers = UNSTUFF_BITS(resp, 122, 4);
		}
		else
		{
			sdhc_version2_struct = UNSTUFF_BITS(resp, 126,2);
			sdhc_mmc_spec_vers = 0;
		}

		if (sdhc_version2_struct)
		{
			ulong csd_c_size;

			/*
			 * Capacity = (C_SIZE+1) * 512K
			 *
			 * NOTE: Adjust calculations by dividing by 512 over the
			 * other calculations so this is capacity in blocks
			 */
			csd_c_size			= UNSTUFF_BITS(resp, 48, 22);
			tran_speed			= UNSTUFF_BITS(resp,  96, 8);

			sdhci_dev->lba = (csd_c_size + 1) << 10;
		}
		else
		{
			ulong csd_c_size;
			ulong csd_c_size_mult;
			ulong csd_read_bl_len;
			ulong mult;
			ulong blocknr;
			ulong block_len;
			
			/*
			 * Capacity = BLOCKNR * BLOCK_LEN
			 *		BLOCKNR = (C_SIZE+1) * MULT
			 *		MULT = 2 ^ (C_SIZE_MULT + 2)	// C_SIZE_MULT < 8
			 *		BLOCK_LEN = 2 ^ READ_BL_LEN		// READ_BL_LEN < 12
			 *
			 * NOTE: Adjust calculations by dividing by 512 over the
			 * other calculations so this is capacity in blocks
			 */
			csd_c_size			= UNSTUFF_BITS(resp,  62, 12);
			csd_c_size_mult		= UNSTUFF_BITS(resp,  47, 3);
			csd_read_bl_len		= UNSTUFF_BITS(resp,  80, 4);
			tran_speed			= UNSTUFF_BITS(resp,  96, 8);

			mult = 1 << (csd_c_size_mult + 2);
			blocknr = (csd_c_size + 1) * mult;
			block_len = 1 << csd_read_bl_len;

			sdhci_dev->lba = (blocknr >> 9) * block_len;
		}
	}
	sdhci_clr_all_int();

	/*
	 * Send Command (SELECT_CARD-7) and wait for response
	 * STANDBY => TRANSFER
	 */
	memset(&mcmd, 0, sizeof(mcmd));
	mcmd.opcode = MMC_SELECT_CARD;
	mcmd.arg = (sdhci_rca << 16); //*sdhci_reg(SDHCI_RESPONSE10_LOFF);
	mcmd.flags = MMC_RSP_R1B | MMC_CMD_AC;
	sdhci_send_cmd(&mcmd);
	if (sdhci_pull_for_int(SDHCI_CMD_COMPLETE))
	{
		printf("SDHCI: Interrupt timeout MMC_SELECT_CARD.\n");
		return;
	}
	sdhci_clr_all_int();

	// Set up transfer modes such as high-speed and 1, 4, 8 bit bus widths for MMC and SD.

	if (if_type != IF_TYPE_MMC)
	{
		int			retries=5;
		int32_t		ints;

		sdhci_clr_all_int();
		sdhci_reset(SDHCI_SW_RESET_CMDLINE);
		sdhci_reset(SDHCI_SW_RESET_DATLINE);

		do
		{

			// Send ACMD51 and wait for response
			memset(&mcmd, 0, sizeof(mcmd));
			mcmd.opcode = MMC_APP_CMD;
			mcmd.arg = sdhci_info.card_rca << 16;
			mcmd.flags = MMC_RSP_R1 | MMC_CMD_AC;
			sdhci_send_cmd(&mcmd);
			if (sdhci_pull_for_int(SDHCI_CMD_COMPLETE))
			{
				puts("SDHCI: Interrupt timeout MMC_APP_CMD.\n");
				ints = -1;
				continue;
			}
			sdhci_clr_all_int();

			memset(&mcmd, 0, sizeof(mcmd));
			mcmd.opcode = SD_APP_SEND_SCR;
			mcmd.flags = MMC_RSP_R1 | MMC_CMD_ADTC;
			// read
			mcmd.dd = 1;
			mcmd.dp = 1;
			mcmd.dma_ena = 1;
			// Set block count to 1, block size to 8
			*sdhci_wreg(SDHCI_BLK_CNT_WOFF) = 1;
			*sdhci_wreg(SDHCI_BLK_SIZE_WOFF) = (SDHCI_HOST_SDMA_BUFFER_SIZE_512K | 8);
			*sdhci_reg(SDHCI_SDMA_SYS_ADDR_LOFF) = ((uint32_t)sdhci_info.sys_buf);
			sdhci_send_cmd(&mcmd);
			if (sdhci_pull_for_int(SDHCI_CMD_COMPLETE))
			{
				puts("SDHCI: Interrupt timeout SD_APP_SEND_SCR.\n");
				ints = -1;
				(void)send_abort_cmd();
				continue;
			}
			if((ints = waitfor_data_transfer_done()) < 0)
			{
				puts("SDHCI: Data transfer interrupt timeout SD_APP_SEND_SCR.\n");
				sdhci_reset(SDHCI_SW_RESET_DATLINE);
				udelay(50);
				(void)send_abort_cmd();
			}
			else if(ints & (SDHCI_DATA_TIMEOUT_ERR |
							SDHCI_DATA_CRC_ERR |
							SDHCI_DATA_END_BIT_ERR))
			{
				puts("Data transfer error SD_APP_SEND_SCR.\n");
				sdhci_reset(SDHCI_SW_RESET_DATLINE);
				udelay(50);
				(void)send_abort_cmd();
			}
		} while(ints && (retries-- != 0));

		sdhci_clr_all_int();
		sdhci_reset(SDHCI_SW_RESET_CMDLINE);
		sdhci_reset(SDHCI_SW_RESET_DATLINE);

		sdhci_info.sd_scr_info.sda_vsn = sdhci_info.sys_buf[0] & 0xF;
		sdhci_info.sd_scr_info.bus_widths = sdhci_info.sys_buf[1] & 0xF;

		printf("Bus Width=0x%X, tran_speed=0x%02X\n",
			   sdhci_info.sd_scr_info.bus_widths,
			   tran_speed);

		if(sdhci_info.sd_scr_info.bus_widths & SD_SCR_BUS_WIDTH_4)
		{
				// Send ACMD6 and wait for response
			memset(&mcmd, 0, sizeof(mcmd));
			mcmd.opcode = MMC_APP_CMD;
			mcmd.arg = sdhci_info.card_rca << 16;
			mcmd.flags = MMC_RSP_R1 | MMC_CMD_AC;
			sdhci_send_cmd(&mcmd);
			if (sdhci_pull_for_int(SDHCI_CMD_COMPLETE))
			{
				puts("SDHCI: Interrupt timeout MMC_APP_CMD.\n");
				return;
			}
			sdhci_clr_all_int();
	
			memset(&mcmd, 0, sizeof(mcmd));
			mcmd.opcode = SD_APP_SET_BUS_WIDTH;
			mcmd.arg = SD_BUS_WIDTH_4;
			mcmd.flags = MMC_RSP_R1 | MMC_CMD_AC;
			sdhci_send_cmd(&mcmd);
			if (sdhci_pull_for_int(SDHCI_CMD_COMPLETE))
			{
				puts("SDHCI: Interrupt timeout SD_APP_SET_BUS_WIDTH.\n");
				return;
			}
			*sdhci_breg(SDHCI_HOST_CTRL_BOFF) |= SDHCI_DATA_TRANSFER_WIDTH_4BIT;
			printf("SDHCI: Card put into 4-bit transfer width mode (%04X/%04X).\n",
				   (*sdhci_reg(SDHCI_RESPONSE10_LOFF)),
				   (unsigned)(*sdhci_wreg(SDHCI_HOST_CTRL_BOFF)));
		}

		sdhci_info.max_clk = 25000000;

		/*
		 * Detect if high-speed supported by this SD card and enable it if so.
		 */

		if(tran_speed == 0x5A)
		{
			int			retries=5;
			int32_t		ints;

			sdhci_clr_all_int();
			sdhci_reset(SDHCI_SW_RESET_CMDLINE);
			sdhci_reset(SDHCI_SW_RESET_DATLINE);

			do
			{
				memset(&mcmd, 0, sizeof(mcmd));
				mcmd.opcode = MMC_SWITCH;
				mcmd.arg = SD_SWITCH_MODE_SET | SD_SWITCH_MODE_HIGHSPEED;
				mcmd.flags = MMC_RSP_R1 | MMC_CMD_ADTC;
				// read
				mcmd.dd = 1;
				mcmd.dp = 1;
				mcmd.dma_ena = 1;
				// Set block count to 1, block size to 8
				*sdhci_wreg(SDHCI_BLK_CNT_WOFF) = 1;
				*sdhci_wreg(SDHCI_BLK_SIZE_WOFF) = (SDHCI_HOST_SDMA_BUFFER_SIZE_512K | 64);
				*sdhci_reg(SDHCI_SDMA_SYS_ADDR_LOFF) = ((uint32_t)sdhci_info.sys_buf);
				sdhci_send_cmd(&mcmd);
				if (sdhci_pull_for_int(SDHCI_CMD_COMPLETE))
				{
					puts("SDHCI: Interrupt timeout MMC_SWITCH (SD).\n");
					ints = -1;
					(void)send_abort_cmd();
					continue;
				}
				if((ints = waitfor_data_transfer_done()) < 0)
				{
					puts("SDHCI: Data transfer interrupt timeout MMC_SWITCH (SD).\n");
					(void)send_abort_cmd();
				}
				else if(ints & (SDHCI_DATA_TIMEOUT_ERR |
								SDHCI_DATA_CRC_ERR |
								SDHCI_DATA_END_BIT_ERR))
				{
					puts("Data transfer error MMC_SWITCH (SD).\n");
					(void)send_abort_cmd();
				}
			} while(ints && (retries-- != 0));

			sdhci_clr_all_int();
			sdhci_reset(SDHCI_SW_RESET_CMDLINE);
			sdhci_reset(SDHCI_SW_RESET_DATLINE);

			if( (sdhci_info.sys_buf[0] != 0) &&
				(sdhci_info.sys_buf[1] != 0) &&
				((sdhci_info.sys_buf[16] & 0x0F) == 1) )
			{
				*sdhci_breg(SDHCI_HOST_CTRL_BOFF) |= SDHCI_HIGH_SPEED_ENABLE;
				puts("SDHCI: Card put into high-speed transfer mode.\n");
				sdhci_info.max_clk = 50000000;
			}
			else
			{
				puts("SDHCI: SD Switch command error, card left in low-speed transfer mode.\n");
			}
		}

#if 0
		// If enable HighSpeed, SD card stops send resp to SET_BLOCK_LEN command.
		// This is caused by FPGA since it runs in Low speed (20Mhz). To be safe,
		// turn off HSpeed in Bootroom and turn it on in Stage_II.
		// Set 4-bit_mode and HighSpeedDisable
		
		if(sdhci_info.sd_scr_info.bus_widths & SD_SCR_BUS_WIDTH_4)
		{
		// Set this SD to 4-bit mode
			*sdhci_breg(SDHCI_HOST_CTRL_BOFF) |= SDHCI_DATA_TRANSFER_WIDTH_4BIT;
		}
		else
		{
		// if the sd card does not support 4bit, set to 1bit mode.
		// Turn off 8-bit and 4-bit
			*sdhci_breg(SDHCI_HOST_CTRL_BOFF) &= ~(SDHCI_DATA_TRANSFER_WIDTH_4BIT | SDHCI_SD_8BIT_MODE);
		}
#endif
		// Set up clocks
	
		sdhci_clr_all_int();
		
		*sdhci_wreg(SDHCI_CLOCK_CTRL_WOFF) = 0;

#if (FPGA_BUILD != NOT_FPGA)
		tmp16 = 0x01;
#else
		if( sdhci_info.max_clk == 25000000 )
			tmp16 = SDHCI_F_OD_HS;
		else if( sdhci_info.max_clk == 50000000 )
			tmp16 = SDHCI_F_OD_VHS;
#endif
		*sdhci_wreg(SDHCI_CLOCK_CTRL_WOFF) = tmp16;
		while ((*sdhci_wreg(SDHCI_CLOCK_CTRL_WOFF)) != (tmp16 | SDHCI_INTERNAL_CLOCK_STABLE))
		{
			udelay(50);
		}
	
#if (FPGA_BUILD != NOT_FPGA)
		tmp16 = 0x05;
#else
		if( sdhci_info.max_clk == 25000000 )
			tmp16 = SDHCI_F_PP_HS;
		else if( sdhci_info.max_clk == 50000000 )
			tmp16 = SDHCI_F_PP_VHS;
#endif
	
		*sdhci_wreg(SDHCI_CLOCK_CTRL_WOFF) = tmp16;
		while ((*sdhci_wreg(SDHCI_CLOCK_CTRL_WOFF)) != (tmp16 | SDHCI_INTERNAL_CLOCK_STABLE))
		{
			udelay(50);
		}

		sdhci_clr_all_int();
		
	}
	else if(if_type == IF_TYPE_MMC)
	{
		int			retries=5;
		int32_t		ints;

		sdhci_clr_all_int();
		sdhci_reset(SDHCI_SW_RESET_CMDLINE);
		sdhci_reset(SDHCI_SW_RESET_DATLINE);

#ifdef HCDEBUG
	dumpHCReg();
#endif

		do
		{
			puts("SDHCI: Getting extended CSD: ");

			memset(&mcmd, 0, sizeof(mcmd));
			mcmd.opcode = MMC_SEND_EXT_CSD;
			mcmd.arg = 0x0;
			mcmd.flags = MMC_RSP_R1 | MMC_CMD_ADTC;
			// read
			mcmd.dd = 1;
			mcmd.dp = 1;
			mcmd.dma_ena = 1;

			// Set block count to 1, block size to 512
			*sdhci_wreg(SDHCI_BLK_CNT_WOFF) = 1;
			*sdhci_wreg(SDHCI_BLK_SIZE_WOFF) = (SDHCI_HOST_SDMA_BUFFER_SIZE_512K | 512);
			*sdhci_reg(SDHCI_SDMA_SYS_ADDR_LOFF) = ((uint32_t)sdhci_info.sys_buf);
			sdhci_send_cmd(&mcmd);
			if (sdhci_pull_for_int(SDHCI_CMD_COMPLETE))
			{
				puts("Interrupt timeout MMC_SEND_EXT_CSD.\n");
				ints = -1;
				(void)send_abort_cmd();
				continue;
			}
			if((ints = waitfor_data_transfer_done()) < 0)
			{
				puts("Data transfer interrupt timeout MMC_SEND_EXT_CSD.\n");
				(void)send_abort_cmd();
			}
			else if(ints & (SDHCI_DATA_TIMEOUT_ERR |
							SDHCI_DATA_CRC_ERR |
							SDHCI_DATA_END_BIT_ERR))
			{
				puts("Data transfer error MMC_SEND_EXT_CSD.\n");
				(void)send_abort_cmd();
			}

//			(void)waitfor_data_transfer_done();
		} while(ints && (retries-- != 0));

		printf("done.\n");

		sdhci_clr_all_int();
		
		sdhci_info.mmc_ext_csd_info.csd_boot_bus_width =
				sdhci_info.sys_buf[EXT_CSD_BOOT_BUS_WIDTH];
		sdhci_info.mmc_ext_csd_info.csd_bus_width =
				sdhci_info.sys_buf[EXT_CSD_BUS_WIDTH];
		sdhci_info.mmc_ext_csd_info.csd_structure_ver =
				sdhci_info.sys_buf[EXT_CSD_STRUCTURE];
		sdhci_info.mmc_ext_csd_info.csd_card_type =
				sdhci_info.sys_buf[EXT_CSD_CARD_TYPE];

		if ( sdhci_hc_card )
		{
			// Update sector count
			uint32_t		*sc = (uint32_t *) &sdhci_info.sys_buf[EXT_CSD_SEC_COUNT];

			sdhci_dev->lba = *sc;
		}

		printf("MMC Spec Version=%d, CSD Card Type=%d for devnum %d\n",
			   sdhc_mmc_spec_vers, sdhci_info.mmc_ext_csd_info.csd_card_type,
			   dev_num);

		/* For the higher spec card, we update the max data transfer rate. */
		if (sdhci_info.sys_buf[EXT_CSD_CARD_TYPE] == EXT_CSD_CARD_TYPE_26MHZ)
		{
			sdhci_info.max_clk = 26000000;
		}
		else if(sdhci_info.sys_buf[EXT_CSD_CARD_TYPE] == EXT_CSD_CARD_TYPE_52MHZ)
		{
			sdhci_info.max_clk = 52000000;
		}
		// The MMC card supports spec 4.0 or higher, we set the clock to 24MHz
		// and set High speed timer in ext_csd for clock > 20MHz.
		// Otherwise to support the very old MMC card, set the clock to 6MHz.
		if ((sdhc_mmc_spec_vers >= 4) &&
            (sdhci_info.mmc_ext_csd_info.csd_card_type == EXT_CSD_CARD_TYPE_52MHZ) &&
		    (dev_num == 0))
		{
			// Send Command 6 and wait for response
			// Set High Speed Timing to 1
			memset(&mcmd, 0, sizeof(mcmd));
			mcmd.opcode = MMC_SWITCH;

			mcmd.arg = (MMC_SWITCH_ACCESS_WRITE_BYTE |
					    (EXT_CSD_HIGH_SPEED_MODE << MMC_SWITCH_INDEX_SHIFT) |
                        (1 << MMC_SWITCH_VALUE_SHIFT));
			mcmd.flags = MMC_RSP_PRESENT | MMC_RSP_R1B;
			if (sdhci_send_cmd_and_retry(&mcmd, 5, SDHCI_CMD_COMPLETE))
			{
				puts("SDHCI: Interrupt timeout MMC_SWITCH high-speed timing.\n");
				sdhci_info.max_clk = 26000000;
			}
			else
			{
				*sdhci_breg(SDHCI_HOST_CTRL_BOFF) |= SDHCI_HIGH_SPEED_ENABLE;
				puts("SDHCI: Card put into high-speed transfer mode.\n");
			}
			sdhci_clr_all_int();
		}
		else
		{
				sdhci_info.max_clk = 26000000;
				puts("SDHCI: Card left in low-speed transfer mode.\n");
		}
		
		// Set up clocks
	
		*sdhci_wreg(SDHCI_CLOCK_CTRL_WOFF) = 0;

#if (FPGA_BUILD != NOT_FPGA)
		tmp16 = 0x01;
#else
		if (sdhc_mmc_spec_vers < 4)
			tmp16 = SDHCI_F_OD_LS;
		else if ((dev_num != 0) || (sdhci_info.max_clk == 26000000))
			tmp16 = SDHCI_F_OD_HS;
		else
			tmp16 = SDHCI_F_OD_VHS;
#endif
		*sdhci_wreg(SDHCI_CLOCK_CTRL_WOFF) = tmp16;
		//outhw(SDIO_CTRL1, tmp16);
		while ((*sdhci_wreg(SDHCI_CLOCK_CTRL_WOFF)) != (tmp16 | SDHCI_INTERNAL_CLOCK_STABLE))
		{
			udelay(50);
		}

#if (FPGA_BUILD != NOT_FPGA)
		tmp16 = 0x05;
#else
		if (sdhc_mmc_spec_vers < 4)
			tmp16 = SDHCI_F_PP_LS;
		else if ((dev_num != 0) || (sdhci_info.max_clk == 26000000))
			tmp16 = SDHCI_F_PP_HS;
		else
			tmp16 = SDHCI_F_PP_VHS;
#endif
	
		*sdhci_wreg(SDHCI_CLOCK_CTRL_WOFF) = tmp16;
		while ((*sdhci_wreg(SDHCI_CLOCK_CTRL_WOFF)) != (tmp16 | SDHCI_INTERNAL_CLOCK_STABLE))
		{
			udelay(50);
		}

		if (sdhc_mmc_spec_vers >= 4)
		{
			if (dev_num != 1)
			{
				// Send Command 6 and wait for response 8 bit mode.
				memset(&mcmd, 0, sizeof(mcmd));
				mcmd.opcode = MMC_SWITCH;
				mcmd.arg = (MMC_SWITCH_ACCESS_WRITE_BYTE |
							(EXT_CSD_BUS_WIDTH << MMC_SWITCH_INDEX_SHIFT) |
							(EXT_CSD_BUS_WIDTH_8 << MMC_SWITCH_VALUE_SHIFT));
				mcmd.flags = MMC_RSP_PRESENT | MMC_RSP_R1B;
				if (sdhci_send_cmd_and_retry(&mcmd, 5, SDHCI_CMD_COMPLETE))
				{
					puts("SDHCI: Interrupt timeout MMC_SWITCH 8-bit mode.\n");
					return;
				}
				sdhci_clr_all_int();
				
				if(((*sdhci_reg(SDHCI_RESPONSE10_LOFF)) & MMC_RSP_CS_SWITCH_ERROR) == 0)
				{
					*sdhci_breg(SDHCI_HOST_CTRL_BOFF) &= ~ (SDHCI_SD_8BIT_MODE | SDHCI_DATA_TRANSFER_WIDTH_4BIT);
					*sdhci_breg(SDHCI_HOST_CTRL_BOFF) |= SDHCI_SD_8BIT_MODE;
					puts("SDHCI: Card put into 8-bit transfer width mode.\n");
				}
				else
				{
					// Send Command 6 and wait for response 4 bit mode.
					memset(&mcmd, 0, sizeof(mcmd));
					mcmd.opcode = MMC_SWITCH;
					mcmd.arg = (MMC_SWITCH_ACCESS_WRITE_BYTE |
							(EXT_CSD_BUS_WIDTH << MMC_SWITCH_INDEX_SHIFT) |
							(EXT_CSD_BUS_WIDTH_4 << MMC_SWITCH_VALUE_SHIFT));
					mcmd.flags = MMC_RSP_PRESENT | MMC_RSP_R1B;
					if (sdhci_send_cmd_and_retry(&mcmd, 5, SDHCI_CMD_COMPLETE))
					{
						puts("SDHCI: Interrupt timeout MMC_SWITCH 4-bit mode.\n");
						return;
					}
					sdhci_clr_all_int();

					// If card status indicates SWITCH command failed then this card doesn't support 4-bit mode.

					if(((*sdhci_reg(SDHCI_RESPONSE10_LOFF)) & MMC_RSP_CS_SWITCH_ERROR) == 0)
					{
						*sdhci_breg(SDHCI_HOST_CTRL_BOFF) &= ~ (SDHCI_SD_8BIT_MODE | SDHCI_DATA_TRANSFER_WIDTH_4BIT);
						*sdhci_breg(SDHCI_HOST_CTRL_BOFF) |= SDHCI_DATA_TRANSFER_WIDTH_4BIT;
						puts("SDHCI: Card put into 4-bit transfer width mode.\n");
					}
					else
					{
						puts("SDHCI: Card left in 1-bit transfer width mode.\n");
					}
				}
			}
			else
			{
					// Send Command 6 and wait for response 4 bit mode.
				memset(&mcmd, 0, sizeof(mcmd));
				mcmd.opcode = MMC_SWITCH;
				mcmd.arg = (MMC_SWITCH_ACCESS_WRITE_BYTE |
						(EXT_CSD_BUS_WIDTH << MMC_SWITCH_INDEX_SHIFT) |
						(EXT_CSD_BUS_WIDTH_4 << MMC_SWITCH_VALUE_SHIFT));
				mcmd.flags = MMC_RSP_PRESENT | MMC_RSP_R1B;
				if (sdhci_send_cmd_and_retry(&mcmd, 5, SDHCI_CMD_COMPLETE))
				{
					puts("SDHCI: Interrupt timeout MMC_SWITCH 4-bit mode.\n");
					return;
				}
				sdhci_clr_all_int();
				
				if(((*sdhci_reg(SDHCI_RESPONSE10_LOFF)) & MMC_RSP_CS_SWITCH_ERROR) == 0)
				{
					*sdhci_breg(SDHCI_HOST_CTRL_BOFF) &= ~ (SDHCI_SD_8BIT_MODE | SDHCI_DATA_TRANSFER_WIDTH_4BIT);
					*sdhci_breg(SDHCI_HOST_CTRL_BOFF) |= SDHCI_DATA_TRANSFER_WIDTH_4BIT;
					puts("SDHCI: Card put into 4-bit transfer width mode.\n");
				}
				else
				{
					puts("SDHCI: Card left in 1-bit transfer width mode.\n");
				}
			}
		}
	}

	/*
	 * Send command (SET_BLOCKLEN-16) and wait for response
	 * TRANSFER => TRANSFER
	 */
	memset(&mcmd, 0, sizeof(mcmd));
	mcmd.opcode = MMC_SET_BLOCKLEN;
	mcmd.arg = sdhci_dev->blksz;
	mcmd.flags = MMC_RSP_R1 | MMC_CMD_AC;
	if (sdhci_send_cmd_and_retry(&mcmd, 5, SDHCI_CMD_COMPLETE))
	{
		printf("SDHCI: Interrupt timeout MMC_SET_BLOCKLEN.\n");
		return;
	}
	sdhci_clr_all_int();

	/*
	 * Mark the device type now that we know for sure what it is
	 */
	sdhci_dev->if_type = if_type;
	sdhci_dev->type = DEV_TYPE_HARDDISK;

	/*
	 * Mark us officially open for business
	 */
	sdhci_ready = 1;

	/*
	 * See if there is a file system on this device
	 */
	init_part(sdhci_dev);
}

ulong
/****************************************************/
sdhci_read(int dev_num, lbaint_t blknr, ulong blkcnt, void *dst)
/****************************************************/
{
    int retries;
	ulong src;

	/*
	 * If there is nothing to transfer, then we are done
	 */
	if (blkcnt == 0)
	{
		return 0;
	}

	/*
	 * Get base addresses for this device
	 */
	sdhci_dev = sdhci_get_dev(dev_num);
	if (!sdhci_dev || !sdhci_ready)
	{
		printf("SDHCI: %s - device is not ready.\n", __FUNCTION__);
		return 0;
	}
	sdhci_base = (ulong)sdhci_dev->priv;

	retries=10;
	do
	{
		/*
		* Calculate byte card offset and byte transfer count
		*/
		src = blknr * sdhci_dev->blksz;
	
		/*
		* Transfer data from the card to memory
		*/

		if (!sdhci_mblock_read(blkcnt, (uchar*)dst, src))
		{
			return blkcnt;
		}
	} while (--retries != 0);

	return 0;
//	if (sdhci_mblock_read(blkcnt, (uchar*)dst, src))
//	{
//		return 0;
//	}
//	return blkcnt;
}

ulong
/****************************************************/
sdhci_write(int dev_num, lbaint_t blknr, ulong blkcnt, const void *src)
/****************************************************/
{
	ulong dst;

	/*
	 * If there is nothing to transfer, then we are done
	 */
	if (blkcnt == 0)
	{
		return 0;
	}

	/*
	 * Get base addresses for this device
	 */
	sdhci_dev = sdhci_get_dev(dev_num);
	if (!sdhci_dev || !sdhci_ready)
	{
		printf("SDHCI: %s - device is not ready.\n", __FUNCTION__);
		return 0;
	}
	sdhci_base = (ulong)sdhci_dev->priv;

	/*
	 * Calculate byte card offset and byte transfer count
	 */
	dst = blknr * sdhci_dev->blksz;

	/*
	 * Transfer data from the card to memory
	 */
	if (sdhci_mblock_write(blkcnt, dst, (uchar*)src))
	{
		return 0;
	}
	return blkcnt;
}


void
/****************************************************/
sdhci_display_capabilites(int dev_num)
/****************************************************/
{
	ulong capable;

	/*
	 * Get base addresses for this device
	 */
	sdhci_dev = sdhci_get_dev(dev_num);
	if (!sdhci_dev || !sdhci_ready)
	{
		puts("    device is not ready\n");
		return;
	}
	sdhci_base = (ulong)sdhci_dev->priv;

	/*
	 * Capabilities
	 * 05:00	Timeout Frequency
	 * 07		Timeout Unit (1=MHZ)
	 * 13:08	Base Clock Frequency
	 * 17:16	Max Block (0=512, 1=1024, 2=2048, 3=4096)
	 * 18		Extended Media Bus Support
	 * 19		ADMA2 Support
	 * 21		High Speed Support
	 * 22		SDMA Support
	 * 23		Suspend/Resume
	 * 24		3.3 Volt
	 * 25		3.0 Volt
	 * 26		1.8 Volt
	 * 27		Interrupt Mode
	 * 28		64 Bit Mode
	 * 29		SPI Mode
	 * 30		SPI Block Mode
	 */
	capable = *sdhci_reg(SDHCI_CAPABILITIES_LOFF);
	{
		uint32_t timeout_freq;
		timeout_freq = (capable >> 0) & 0x3F;
		printf("    TimeOut Clock: %d ", timeout_freq);
	}
	{
		ulong timeout_unit;
		timeout_unit = (capable >> 7) & 0x01;
		puts((timeout_unit) ? "M" : "K");
		puts("Hz");
	}
	{
		uint32_t base_freq;
		base_freq = (capable >> 8) & 0x3F;
		printf("    Base Clock: %d MHz", base_freq);
	}
	puts("\n");
	{
		uint32_t max_block;
		max_block = 512 << ((capable >> 16) & 0x03);
		printf("    MaxBlockLength: %d", max_block);
	}
	puts("   ");
	{
		ulong volt3_3_support;
		volt3_3_support = (capable >> 24) & 1;
		if (volt3_3_support)
			puts(" 3.3 Volt");
	}
	{
		ulong volt3_0_support;
		volt3_0_support = (capable >> 25) & 1;
		if (volt3_0_support)
			puts(" 3.0 Volt");
	}
	{
		ulong volt1_8_support;
		volt1_8_support = (capable >> 26) & 1;
		if (volt1_8_support)
			puts(" 1.8 Volt");
	}
	puts("\n");
	puts("   ");
	{
		ulong adma2_support;
		adma2_support = (capable >> 19) & 0x01;
		if (adma2_support)
			puts(" ADMA2");
	}
	{
		ulong sdma_support;
		sdma_support = (capable >> 22) & 1;
		if (sdma_support)
			puts(" SDMA");
	}
	{
		ulong high_speed_support;
		high_speed_support = (capable >> 21) & 0x01;
		if (high_speed_support)
			puts(" HighSpeed");
	}
	{
		ulong ext_media_bus_support;
		ext_media_bus_support = (capable >> 18) & 0x01;
		if (ext_media_bus_support)
			puts(" ExtMediaBus");
	}
	{
		ulong suspend_resume_support;
		suspend_resume_support = (capable >> 23) & 1;
		if (suspend_resume_support)
			puts(" Suspend/Resume");
	}
	puts("\n");
	puts("   ");
	{
		ulong interrupt_mode_support;
		interrupt_mode_support = (capable >> 27) & 1;
		if (interrupt_mode_support)
			puts(" Interrupt");
	}
	{
		ulong bit64_mode_support;
		bit64_mode_support = (capable >> 28) & 1;
		if (bit64_mode_support)
			puts(" 64-bitSystemBus");
	}
	{
		ulong spi_mode_support;
		spi_mode_support = (capable >> 29) & 1;
		if (spi_mode_support)
			puts(" SPI");
	}
	{
		ulong spi_block_mode_support;
		spi_block_mode_support = (capable >> 30) & 1;
		if (spi_block_mode_support)
			puts(" SPI-Block");
	}
	puts("\n");
}

#endif	/* CONFIG_CMD_SDHCI */
