/*
 * BCM4760 'flipflop' - Retains 1-bit across a watchdog reset.
 *
 * Copyright (C) 2010 TomTom International B.V.
 * Author: Martin Jackson <martin.jackson@tomtom.com>
 * 
 ************************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 
 * 02110-1301, USA.
 ************************************************************************
 */

/* #define DEBUG */

#include <common.h>
#include <asm/io.h>
#include <asm/arch/bcm4760_reg.h>
#include <linux/bitops.h>

#define BBL_SW_REG3	0x29

#define FLIPFLOP_ADDR PML_R_PML_CTL0_MEMADDR
#define FLIPFLOP_BIT 30

#ifdef CONFIG_SW_FLIPFLOP
static inline void bbl_wait(void)
{
        volatile u32 *io_ctl_addr = (u32 *)(RTC_R_BLKCTL_MEMADDR);
	volatile u32 value;

        do
        {
                value = *io_ctl_addr;
        } while (value & 0x01 );
}
#endif

#ifdef CONFIG_SW_FLIPFLOP
void flipflop_set(int v)
{
        volatile u32 * io_sts_addr = (u32 *)(RTC_R_BLKSTS_MEMADDR);
        volatile u32 * io_ctl_addr = (u32 *)(RTC_R_BLKCTL_MEMADDR);

        bbl_wait();

	// Issuing Address Register Command, bit[0]=GO, bit[1]=Write/Read
	*io_sts_addr = (u32)v;
	*io_ctl_addr = (0x0003 | (BBL_SW_REG3 & 0xff) << 8);
}
#else
void flipflop_set(int v)
{
	unsigned long t;

	debug("Setting flipflop [v=%d]\n", v);

	t = readl(FLIPFLOP_ADDR);
	v? __set_bit(FLIPFLOP_BIT, &t) : __clear_bit(FLIPFLOP_BIT, &t);
	writel(t, FLIPFLOP_ADDR);
}
#endif

#ifdef CONFIG_SW_FLIPFLOP
int flipflop_get(void)
{
        volatile u32 * io_sts_addr = (u32 *)(RTC_R_BLKSTS_MEMADDR);
        volatile u32 * io_ctl_addr = (u32 *)(RTC_R_BLKCTL_MEMADDR);
        volatile u32 val;

        bbl_wait();

        *io_ctl_addr = (0x0001 | (BBL_SW_REG3 & 0xff) << 8);
        bbl_wait();

	val = *io_sts_addr;

        return val;
}
#else
int flipflop_get(void)
{
	unsigned long t;
	int r;

	t = readl(FLIPFLOP_ADDR);
	r = test_bit(FLIPFLOP_BIT, &t);

	debug("Flipflop returned [%d]\n", r);

	return r;
}
#endif

