/*****************************************************************************
*  Copyright 2008 Broadcom Corporation.  All rights reserved.
*
*  Unless you and Broadcom execute a separate written software license
*  agreement governing use of this software, this software is licensed to you
*  under the terms of the GNU General Public License version 2, available at
*  http://www.gnu.org/copyleft/gpl.html (the "GPL").
*
*  Notwithstanding the above, under no circumstances may you combine this
*  software in any way with any other Broadcom software provided under a
*  license other than the GPL, without Broadcom's express prior written
*  consent.
*
*****************************************************************************/

#ifndef __BOARD_CONFIG_H
#define __BOARD_CONFIG_H

//#define KOSTCO

/*
 * High Level Configuration Options
 */
#define CONFIG_ARM1136			1	/* This is an arm1136 CPU core */
#define CONFIG_BCM476X
#define CONFIG_BCM4760

#include <asm/arch/bcm4760.h>			/* get chip and board defs */

/* Enable/Disable secure u-boot */
#define CONFIG_SIGNATURE		1

/* input clock of PLL */
#define CONFIG_SYS_CLK_FREQ		0
#ifdef KOSTCO
#define UARTCLK					( 30000000 )
#else
#define UARTCLK					( 78000000 )
#endif

#undef  CONFIG_USE_IRQ					/* no support for IRQs */
#define CONFIG_MISC_INIT_R

#define CONFIG_CMDLINE_TAG		1	/* enable passing of ATAGs */
#define CONFIG_SETUP_MEMORY_TAGS	1
#define CONFIG_INITRD_TAG		1
#define CONFIG_REVISION_TAG		0


/*-----------------------------------------------------------------------
 * Physical Memory Map
 */
#define CONFIG_NR_DRAM_BANKS	1
#define PHYS_SDRAM_1			HW_SDRAM_BASE
#define PHYS_SDRAM_1_SIZE		HW_SDRAM_SIZE

/*-----------------------------------------------------------------------
 * Stack sizes
 *
 * The stack sizes are set up in start.S using the settings below
 */
#define CONFIG_STACKSIZE		SZ_128K	/* regular stack */
#ifdef CONFIG_USE_IRQ
 #define CONFIG_STACKSIZE_IRQ		SZ_4K	/* IRQ stack */
 #define CONFIG_STACKSIZE_FIQ		SZ_4K	/* FIQ stack */
#endif

/*
 * Size of malloc() pool
 */
#define CFG_ENV_SIZE			SZ_128K
#define CFG_MALLOC_LEN			(CFG_ENV_SIZE + SZ_128K)
#define CFG_GBL_DATA_SIZE		128	/* bytes reserved for initial data */

/*
 * Hardware drivers
 */
#if 0
/**
 ** I could not get the NS16550 driver to work, so went with a special driver
 ** to make bring up faster... but this should probably get moved back if time
 ** permits... [denis]
 **/
/*
 * NS16550 Configuration
 */
#define CFG_NS16550
#define CFG_NS16550_SERIAL
#define CFG_NS16550_REG_SIZE	(-4)
#define CFG_NS16550_CLK			UARTCLK
#define CFG_NS16550_COM1		HW_UART_A_BASE

#else
/*
 * UART Configuration
 */
#define CFG_BCM4760_UART
#define CFG_BCM4760_UART_CLK	UARTCLK
#define CFG_BCM4760_UART_COM	HW_UART_A_BASE

#endif

/*
 * select serial console configuration
 */
#define CONFIG_SERIAL1			1	/* UART1 */
#define CONFIG_SERIAL2			2	/* UART2 */
#define CONFIG_SERIAL3			3	/* UART3 */

#define CFG_CONSOLE_INFO_QUIET	0

/* allow to overwrite serial */
#define CONFIG_ENV_OVERWRITE
#define CONFIG_CONS_INDEX		1
#define CONFIG_BAUDRATE			115200
#define CFG_BAUDRATE_TABLE		{9600, 19200, 38400, 57600, 115200}


/* Boot delay */
#define CONFIG_BOOTDELAY		0
#define CONFIG_BOOTCOMMAND		"sd boot"
#define CONFIG_BOOTARGS			"root=/dev/mtdblock0 console=ttyS0,115200 mem=48M ram_mtd_size=16M arm_vc_buf_size=0M loglevel=10"

/*
 * Miscellaneous configurable options
 */
#define CFG_LONGHELP			/* undef to save memory */
#define CFG_PROMPT				"# "
#define CFG_CBSIZE				2048	/* Console I/O Buffer Size */
/* Print Buffer Size */
#define CFG_PBSIZE				(CFG_CBSIZE+sizeof(CFG_PROMPT)+16)
#define CFG_MAXARGS				16	/* max number of command args */
#define CFG_BARGSIZE			CFG_CBSIZE	/* Boot Argument Buffer Size */

#define CFG_MEMTEST_START		(PHYS_SDRAM_1)	/* memtest works on */
#define CFG_MEMTEST_END			(CFG_MEMTEST_START + PHYS_SDRAM_1_SIZE)

#undef	CFG_CLKS_IN_HZ			/* everything, incl board info, in Hz */

/* default load address */
#define CFG_LOAD_ADDR			(PHYS_SDRAM_1 + 0x00800000 - sizeof(image_header_t))
#define CFG_ATAG_ADDR			(PHYS_SDRAM_1 + 0x100)


#define TIMER_BASE				HW_GPTIMER0_BASE
#define CFG_TIMERBASE			24000000UL
#define CFG_HZ					(CONFIG_SYS_CLK_FREQ)

/*-----------------------------------------------------------------------
 * FLASH and environment organization
 */
#define CFG_NO_FLASH
#define CFG_ENV_IS_NOWHERE

/*
 * Primary Storage
 */
#ifdef KOSTCO
#define CFG_SDHCI_MAXDEVICE		1
#else
#define CFG_SDHCI_MAXDEVICE		3
#endif


/***********************************************************
 * Command definition
 ***********************************************************/
#include <config_cmd_default.h>
//#define CONFIG_CMD_AUTOSCRIPT	/* Autoscript Support			*/
//#define CONFIG_CMD_BDI		/* bdinfo						*/
//#define CONFIG_CMD_BOOTD		/* bootd						*/
//#define CONFIG_CMD_CONSOLE	/* coninfo						*/
//#define CONFIG_CMD_ECHO		/* echo arguments				*/
//#define CONFIG_CMD_ENV		/* saveenv						*/
#define	CONFIG_CMD_FAT			/* FAT filesystem				*/
#undef  CONFIG_CMD_FLASH		/* flinfo, erase, protect		*/
//#define CONFIG_CMD_FPGA		/* FPGA configuration Support	*/
//#define CONFIG_CMD_IMI		/* iminfo						*/
#undef  CONFIG_CMD_IMLS			/* List all found images		*/
//#define CONFIG_CMD_ITEST		/* Integer (and string) test	*/
//#define CONFIG_CMD_LOADB		/* loadb						*/
//#define CONFIG_CMD_LOADS		/* loads						*/
//#define CONFIG_CMD_MEMORY		/* md mm nm mw cp cmp crc base loop mtest */
//#define CONFIG_CMD_MISC		/* Misc functions like sleep etc*/
#undef  CONFIG_CMD_NET			/* bootp, tftpboot, rarpboot	*/
#undef	CONFIG_CMD_NFS			/* NFS support					*/
//#define CONFIG_CMD_RUN		/* run command in env variable	*/
#define CONFIG_CMD_SDHCI		/* BRCM SD/SDHC/MMC device		*/
#undef  CONFIG_CMD_SPI
//#define CONFIG_CMD_SETGETDCR	/* DCR support on 4xx			*/
//#define CONFIG_CMD_XIMG		/* Load part of Multi Image		*/



/* Partitions */
#define CONFIG_DOS_PARTITION

/* Watchdog */
//#define CONFIG_HW_WATCHDOG
#define CONFIG_HW_WATCHDOG_LOWLEVEL_INIT
#define CONFIG_HW_WATCHDOG_TIMER_SECONDS		15

#endif /* __BOARD_CONFIG_H */
