/*
 * fdt_boxkey.h - hardcoded device tree blob to pass the encrypted box key to the kernel
 *
 * 2009-11-03 Ard Biesheuvel <ard.biesheuvel@tomtom.com>
 */

#include <fdt.h>

/*

Create something like this in memory and inject the encrypted boxkey later:

00000000  d0 0d fe ed 00 00 00 85  00 00 00 38 00 00 00 74  |...........8...t|
00000010  00 00 00 28 00 00 00 11  00 00 00 10 00 00 00 00  |...(............|
00000020  00 00 00 11 00 00 00 3c  00 00 00 00 00 00 00 00  |.......<........|
00000030  00 00 00 00 00 00 00 00  00 00 00 01 00 00 00 00  |................|
00000040  00 00 00 01 66 65 61 74  75 72 65 73 00 00 00 00  |....features....|
00000050  00 00 00 03 00 00 00 09  00 00 00 00 58 58 58 58  |............XXXX|
00000060  58 58 58 58 00 00 00 00  00 00 00 02 00 00 00 02  |XXXX............|
00000070  00 00 00 09 66 64 74 2d  72 65 63 6f 76 65 72 79  |....fdt-recovery|
00000080  2d 6b 65 79 00                                    |-key.|
00000085

*/

#define NTOHL(a) \
	((a) & 0xff) << 24 | ((a) & 0xff00) << 8 | ((a) & 0xff0000) >> 8 | ((a) & 0xff000000) >> 24

#define KEY_NAME	"fdt-recovery-key"
#define KEY_SIZE	512

static struct _pbk_fdt
{
	struct fdt_header		header;
	struct fdt_reserve_entry	reserve_entry;

	uint32_t			rootnode;
	uint32_t			rootval;

	uint32_t			featurenode;
	unsigned char			featureval[12];

	uint32_t			tag1;
	uint32_t			len1;
	uint32_t			off;
	unsigned char			key[KEY_SIZE];

	uint32_t			featureendtag;
	uint32_t			rootendtag;
	uint32_t			endval;

	unsigned char			strings[sizeof(KEY_NAME)];

} const pbk_fdt = {
	{
	  NTOHL(FDT_MAGIC),

	  NTOHL(sizeof(struct _pbk_fdt)),		/* total size */

	  NTOHL(sizeof(struct fdt_header) +
		sizeof(struct fdt_reserve_entry)),	/* offset to structure block */

	  NTOHL(0x68 + KEY_SIZE),			/* offset to strings block */

	  NTOHL(sizeof(struct fdt_header)),		/* offset to reserve_entry */

	  NTOHL(0x11), 					/* this version */

	  NTOHL(0x10),					/* last compatible version */

	  0x0,						/* boot cpu */

	  NTOHL(sizeof(KEY_NAME)),			/* size of the strings block */

	  NTOHL(0x26 + KEY_SIZE),			/* size of the structure block */
	},

	{ },

	NTOHL(FDT_BEGIN_NODE), 0x0,

	NTOHL(FDT_BEGIN_NODE), { 'f', 'e', 'a', 't', 'u', 'r', 'e', 's', 0, 0, 0, 0 },

	NTOHL(FDT_PROP), NTOHL(KEY_SIZE), 0x0, { },

	NTOHL(FDT_END_NODE), NTOHL(FDT_END_NODE), NTOHL(FDT_END),

	KEY_NAME
};

