#include <fcntl.h>
/* #include <stdio.h> */
#include <string.h>
/* #include <sys/types.h> */
#include <sys/stat.h>
#include <unistd.h>

/* #include <asm/types.h> */
/* #include <image.h> */

#include <openssl/ssl.h>
//#include <openssl/rsa.h>
#include <openssl/err.h>
#include <openssl/pem.h>
#include <openssl/sha.h>

#define PRIVATE_KEY	"/etc/ssl/uboot"


void SSL_load_error_strings(void);

static DSA *load_key(void) ;
static char *get_key_file_name(void);
static FILE *open_key_file(char *filename);
static DSA *get_private_key(FILE * fp);
static size_t sign_digest(const unsigned char *digest,
			  DSA * private_key,
			  unsigned char *signature);

/*
 =========================================================================
*/

static void
err_msg(void)
{
	char err_buff[1024];
	unsigned long err;

	SSL_load_error_strings();
	
	while ((err = ERR_get_error())) {
		ERR_error_string_n(err, err_buff, sizeof(err_buff));
		fprintf(stderr, "%s\n", err_buff);
	}

	ERR_free_strings();
}  


size_t
get_signature(const unsigned char *buf,
	      unsigned int len,
	      unsigned char *signature)
{
	DSA *private_key;
	unsigned char digest[SHA_DIGEST_LENGTH];
	size_t sign_len;

	/* Do SHA1 encryption of buffer */
	(void)SHA1(buf, len, digest);

	/* Load private key */
	if ((private_key = load_key()) == NULL) {
		return 0;
	}

	/* Sign digest */
	sign_len = sign_digest(digest, private_key, signature);

	/* Clean up */
	DSA_free(private_key);

	return sign_len;
}


static DSA *
load_key(void) 
{
	DSA *key;
	char *key_name;
	FILE *fp;
	int res;

	key_name = get_key_file_name();
	if (key_name == NULL) {
		return NULL;
	}

	fp = open_key_file(key_name);
	if (fp == NULL) {
		free(key_name);
		return NULL;
	}

	free(key_name);

	key = get_private_key(fp);

	if (key == NULL) {
		return NULL;
	}

	res = fclose(fp);
	if (res) {
		fprintf(stderr, "Can't close file: %s\n", strerror(errno));
		DSA_free(key);
		key = NULL;
	}
	
	return key;
}


static char *
get_key_file_name(void)
{
	char *file_name = NULL;

	file_name = malloc(strlen(PRIVATE_KEY));
	strncpy(file_name, PRIVATE_KEY, strlen(PRIVATE_KEY));

	return file_name;
}


static FILE *
open_key_file(char *file_name)
{
	FILE *fp;
	int fd;
 	struct stat st;

	/* Open key and check size of file */
	fd = open(file_name, O_RDONLY);
	if (fstat(fd, &st) < 0 ) {
		fprintf(stderr, "fstat for key file %s failed: %s\n",
			file_name, strerror(errno));
		return NULL;
	}

	if (st.st_size > 1 * 1024 * 1024) {
		fprintf(stderr, "key file %s is too large\n", file_name);
		return NULL;
	}

	fp = fdopen(fd, "r");
	if (fp == NULL) {
		fprintf(stderr, "Can't fdopen file: %s\n", strerror(errno));
		return NULL;
	}
	
	return fp;
}


static DSA *
get_private_key( FILE *fp)
{
	DSA *key;
#if (0) /* for RSA only */	
	int res;
#endif 	

	/* Load private key */
	key = PEM_read_DSAPrivateKey(fp, NULL, NULL, NULL);
	
#if (0) /* for RSA only */	
	res = DSA_check_key(key);
	if (res == 0) {
		DSA_free(key);
		key = NULL;
	}
#endif 	

	if (key == NULL) {
		err_msg();
	}
	
	return key;
}


/* Do SHA1 encryption of buffer.  Result is in digest*/
static size_t
sign_digest(const unsigned char *digest,
	    DSA * private_key,
	    unsigned char *signature)
{
	unsigned int sign_len;
	int res;
	DSA_SIG	*s1;
	const unsigned char *signature_p = signature;

	s1 = DSA_SIG_new();
	
	/* Allocate space for signature */
	sign_len = DSA_size(private_key);

	/* Sign digest */
	res = DSA_sign(NID_sha1, digest, SHA_DIGEST_LENGTH,
		       signature, &sign_len, private_key);
	if (res == 0) {
		err_msg();
		sign_len = 0;
	}

	d2i_DSA_SIG(&s1, &signature_p, sign_len);
	sign_len = BN_bn2bin(s1->r, signature);
	sign_len += BN_bn2bin(s1->s, signature+sign_len);

	DSA_SIG_free(s1);
	
	return sign_len;
}
